<div id="toast-clipboard-container" class="fixed-top ml-30 p-3">
    <div id="toast-clipboard-toast" class="toast" role="alert"
         aria-atomic="tue"
         aria-live="polite">
        <div class="toast-body text-success">
            <div>
                <i class="bi bi-clipboard-check"></i>
                Copied <span id="clipboard-copy-target"></span> to clipboard!
            </div>
            <div>
                <code id="clipboard-copy-value">
                </code>
            </div>
        </div>
    </div>
</div>
