{# Index of template variables:
       project: str - The name of the project being tested
       report_title: str - The title of the report that was generated
#}
<!DOCTYPE html>
<html>
<head>
    <meta charset="utf-8">
    <link href="https://cdn.jsdelivr.net/npm/bootstrap@5.1.3/dist/css/bootstrap.min.css"
         rel="stylesheet"
         integrity="sha384-1BmE4kWBq78iYhFldvKuhfTAU6auU8tT94WrHftjDbrCEXSU1oBoqyl2QvZ6jIW3"
         crossorigin="anonymous">
    <link rel="stylesheet"
          href="https://cdn.jsdelivr.net/npm/bootstrap-icons@1.7.2/font/bootstrap-icons.css">
    <link rel="stylesheet" href="static/style.css">
    <script src="https://cdn.jsdelivr.net/npm/bootstrap@5.1.3/dist/js/bootstrap.bundle.min.js"
            integrity="sha384-ka7Sk0Gln4gmtz2MlQnikT1wXgYsOg+OMhuP+IlRH9sENBO0LRn5q+8nbTov4+1p"
            crossorigin="anonymous"></script>
    <script lang="javascript" src="static/report.js"></script>
    <title>{{ report.title }}</title>
</head>
<body>
<div class="container" id="content">
    {% include 'report_header.html' %}
    <div id="results">
        {% for result in report.results %}
            {% include 'test_case.html' %}
        {% endfor %}
    </div>
</div>
{% include 'toast-copied-to-clipboard.html' %}
{% include 'help.html' %}
</body>
</html>
